package dev.svitan.schemas

import kotlinx.serialization.Serializable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.UUID

enum class ActionKind {
    TEXT, SCRIPT;

    companion object {
        fun from(it: String): ActionKind {
            return when (it.lowercase()) {
                "text" -> ActionKind.TEXT
                "script" -> ActionKind.SCRIPT
                else -> throw IllegalArgumentException("Invalid action kind: $it")
            }
        }
    }

    override fun toString(): String {
        return super.toString().lowercase()
    }
}

@Serializable
class ActionDTO(
    val id: String,
    val name: String,
    val kind: ActionKind,
    val source: String,
    val authId: String,
    val createdAt: String,
    val updatedAt: String
)

@Serializable
class NewActionDTO(
    val name: String, val kind: String, val source: String, val authId: String
)

class ActionService {
    object Actions : Table("actions") {
        val id = uuid("id").autoGenerate()
        val name = text("name").uniqueIndex()
        val kind = text("kind")
        val aSource = text("source") // `source` is another property
        val auth = uuid("auth_id").references(AuthService.Auths.id)
        val createdAt = text("created_at")
        val updatedAt = text("updated_at")

        override val primaryKey = PrimaryKey(id)
    }

    companion object {
        fun init() {
            transaction {
                SchemaUtils.create(Actions)
            }
        }

        fun create(action: NewActionDTO): UUID = transaction {
            val now = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
            val authId = UUID.fromString(action.authId)
            AuthService.read(authId) ?: throw Exception("auth not found")

            Actions.insert {
                it[name] = action.name
                it[kind] = ActionKind.from(action.kind).toString()
                it[aSource] = action.source
                it[auth] = authId
                it[createdAt] = now
                it[updatedAt] = now
            }[Actions.id]
        }

        fun read(id: UUID): ActionDTO? = transaction {
            Actions.selectAll()
                .where { Actions.id eq id }
                .map {
                    ActionDTO(
                        it[Actions.id].toString(),
                        it[Actions.name],
                        ActionKind.from(it[Actions.kind]),
                        it[Actions.aSource],
                        it[Actions.auth].toString(),
                        it[Actions.createdAt],
                        it[Actions.updatedAt]
                    )
                }.singleOrNull()
        }

        fun readAll(): List<ActionDTO> = transaction {
            Actions.selectAll().map {
                ActionDTO(
                    it[Actions.id].toString(),
                    it[Actions.name],
                    ActionKind.from(it[Actions.kind]),
                    it[Actions.aSource],
                    it[Actions.auth].toString(),
                    it[Actions.createdAt],
                    it[Actions.updatedAt]
                )
            }
        }

        fun update(id: UUID, action: NewActionDTO) {
            transaction {
                val now = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
                Actions.update({ Actions.id eq id }) {
                    it[name] = action.name
                    it[kind] = ActionKind.from(action.kind).toString()
                    it[aSource] = action.source
                    it[updatedAt] = now
                }
            }
        }

        fun delete(id: UUID) {
            transaction {
                Actions.deleteWhere { Actions.id eq id }
            }
        }
    }
}
